clear all

mex geoTransform.c
mex CtestEstep.c
posFolder = 'cap';
% CONSTANTS %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
scale = .7;  % scale of Gabors, length = 17 pixels
norient = 15;  % number of orientations
T = 0.;        % threshold for responses
sx = 100; sy = 100;
% get gabor responses
[allfiltered allsymbol allfilter images] = getGaborResponse(posFolder,T,scale,norient,sx,sy);
nimage = size(allfiltered,1);

% initially examples are assigned to the first orientation of the template (to be learned) 
dataWeights = ones(nimage,norient);
for i = 1:nimage
    dataWeights(i,:) = dataWeights(i,:) / sum(dataWeights(i,:));
end

step = 1;
while step<=20
    tic
    % prepare space for the Gabor responses, and make a copy of S1 map at all rotation angles
    rotatedAngles = 0:-2:-2*(norient-1); % the rotation of image is opposite to that of templates
    allfiltered2 = cell(nimage*length(rotatedAngles),norient);
    for iRotation = 1:length(rotatedAngles) % rotation
        for i = 1:nimage
            for j = 1:norient % orientation
                tmp = allfiltered{i,j} + 0;
                j2 = j + rotatedAngles(iRotation);
                while j2 > norient
                    j2 = j2 - norient;
                end
                while j2 < 1
                    j2 = j2 + norient;
                end
                allfiltered2{(iRotation-1)*nimage+i,j2} = imrotate(tmp,rotatedAngles(iRotation)*180/norient,'crop');
            end
        end
    end
    [learnedModel Asyml]= ABlearn(reshape(dataWeights,1,nimage*length(rotatedAngles)),allfiltered2,allsymbol,allfilter,T);
    clear allfiltered2
    disp(['training time (M-step): ' num2str(toc) ' seconds']);
    tic
    destFolder = [];
    [tss Asym shiftX shiftY rotation] = ABtest_one(learnedModel,allsymbol,allfilter,allfiltered);
    disp(['testing time (E-step): ' num2str(toc) ' seconds']);
    dif = 1;
    if step > 1
        dif = 0;
        dif = dif + sum((dataWeightsLastTime(:)-dataWeights(:)).^2);
        dif = sqrt(dif)
    end
    if step>1 && dif < 1e-6
        break
    end

    % update data weights
    dataWeightsLastTime = dataWeights;
    a = sum(dataWeights,1);
    div = exp(tss(:,1)) * a(1);
    for o1 = 2:norient
        div = div + exp(tss(:,o1)) * a(o1);
    end
    for o = 1:norient
        dataWeights(:,o) = exp(tss(:,o)) * a(o) ./ div;
    end
    save(sprintf('model%d.mat',step),'learnedModel', 'dataWeights', 'shiftX', 'shiftY', 'rotation');
    imwrite(1-learnedModel.sym,sprintf('t%d.png',step));
    
    Asyml2 = cell(nimage,1);
    for i = 1:nimage
        Asyml2{i} = zeros(size(Asyml{1,1}));
    end

    for i = 1:nimage
        [tmp o] = max(tss(i,:));
        imTransform(1,norient, Asyml((o-1)*nimage+i), Asyml2(i), sx, sy,...
            -shiftX(i), -shiftY(i), -rotation(i));
    end
    im = displayImagesAndSketches(images, Asyml2, 5, sx, sy);
    outepsgif(im, ['ex_trans_' posFolder '_results' num2str(step)]); 
    im = displayImagesSymAndSketches(images, learnedModel.sym, Asyml2, 6, sx, sy);
    close
    step = step + 1;
end



